package tools.descartes.dni.dnimm3.tools;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.Collections;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import tools.descartes.dni.dnimm3.DNIPackage;
import tools.descartes.dni.dnimm3.NetworkInfrastructure;
import tools.descartes.dni.dnimm3.impl.NetworkInfrastructureImpl;

public class DNIParser {

	private DNIParser() {
	}

	public static NetworkInfrastructureImpl convertToModel(File file) {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());
		resourceSet.getPackageRegistry().put(DNIPackage.eNS_URI, DNIPackage.eINSTANCE);
		Resource resource = resourceSet.getResource(URI.createFileURI(file.getAbsolutePath()), true);
		return (NetworkInfrastructureImpl) resource.getContents().get(0);
	}

	public static NetworkInfrastructureImpl convertToModel(byte[] data) throws IOException {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());
		resourceSet.getPackageRegistry().put(DNIPackage.eNS_URI, DNIPackage.eINSTANCE);
		Resource resource = resourceSet.createResource(URI.createURI(""));
		resource.load(new ByteArrayInputStream(data), Collections.EMPTY_MAP);
		return (NetworkInfrastructureImpl) resource.getContents().get(0);
	}

	public static byte[] convertToByteArray(NetworkInfrastructure networkInfrastructure) throws IOException {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());
		resourceSet.getPackageRegistry().put(DNIPackage.eNS_URI, DNIPackage.eINSTANCE);
		Resource resource = resourceSet.createResource(URI.createURI(""));
		resource.getContents().add(networkInfrastructure);
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		resource.save(baos, Collections.EMPTY_MAP);
		return baos.toByteArray();
	}

	public static void writeToFile(NetworkInfrastructure networkInfrastructure, File file) throws IOException {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());
		resourceSet.getPackageRegistry().put(DNIPackage.eNS_URI, DNIPackage.eINSTANCE);
		Resource resource = resourceSet.createResource(URI.createFileURI(file.getAbsolutePath()));
		resource.getContents().add(DNIUtil.cloneModel(networkInfrastructure));
		resource.save(Collections.EMPTY_MAP);
	}

}
